class YattController < ApplicationController
  unloadable

  before_filter :get_plugin_version
  before_filter :find_project, :only => [:project_time_entries, :project_activities]
  accept_api_auth :global_activities, :project_activities, :all_time_entries, :project_time_entries


  def global_activities
    begin
      @activities = TimeEntryActivity.shared.active.find(:all)
      respond_to { |format|
        format.api { render :activities }
      }
    rescue
      render_400
    end

  end

  def project_activities
    begin
      @activities = @project.activities
      respond_to { |format|
        format.api { render :activities }
      }
    rescue
      render_400
    end
  end

  def all_time_entries
    begin
      @time_entries = format get_time_entries
      respond_to { |format| format.api { render 'time_entries'} }
    rescue
      render_400
    end

  end

  def project_time_entries
    begin
      @time_entries = format get_time_entries
      respond_to { |format| format.api { render 'time_entries' } }
    rescue
      render_400
    end
  end

  private

  def format(time_entries)
    response = {
      :time_entries => [],
      :issues => {},
      :projects => {}
    }
    time_entries.each { |entry|
      response[:time_entries].push entry
      if entry.issue_id and not response[:issues][entry.issue_id]
        response[:issues][entry.issue_id] = entry.issue
      end
      if not response[:projects][entry.project_id]
        response[:projects][entry.project_id] = entry.project
        p entry.project
      end  
    }
    response
  end
  
  def get_time_entries
    @limit = params[:limit].to_i
    @limit = 500 unless (1..500).member? @limit
    @offset = params[:offset].to_i

    user_id = params[:user_id] || User.current.id

    conditions = ['user_id = :user_id', {:user_id => user_id}]
    if (defined? @project)
      conditions[0] += ' AND project_id = :project_id'
      conditions[1][:project_id] = @project.id
    end

    if (params[:date_from])
      date_from = Date.parse params[:date_from]
      conditions[0] += ' AND spent_on >= :date_from'
      conditions[1][:date_from] = date_from
    end

    if (params[:date_to])
      date_to = Date.parse params[:date_to]
      conditions[0] += ' AND spent_on <= :date_to'
      conditions[1][:date_to] = date_to
    end

    order = 'spent_on DESC'
    if (params[:order_by])
      order = params[:order_by].split('.').join(' ')
    end

    @count = TimeEntry.count(:conditions => conditions)

    return TimeEntry.find(
      :all,
      :conditions => conditions,
      :limit => @limit,
      :offset => @offset,
      :order => order,
      :include => [:issue, :project]
    )
  end

  def find_project
    if (issue_id = (params[:issue_id] || params[:time_entry] && params[:time_entry][:issue_id])).present?
      @issue = Issue.find(issue_id)
      @project = @issue.project
    elsif (project_id = (params[:project_id] || params[:time_entry] && params[:time_entry][:project_id])).present?
      @project = Project.find(project_id)
    else
      render_404
      return false
    end
  rescue ActiveRecord::RecordNotFound
    render_404
  end

  def render_400
    render_error(:status => 400, :message => 'Bad request')
    return false
  end

  def get_plugin_version
    @version = Redmine::Plugin.find(:redmine_yatt_backend).version
  end

end
